///////////////////////////////////////////////////////////////////////////////
// $Id: M68681.hxx,v 1.2 1995/01/13 00:32:06 bmott Exp $
///////////////////////////////////////////////////////////////////////////////
// M68681.hxx
//
// The Motorola 68681 DUART
//
// Sim68000 "Motorola 68000 Simulator"
// Copyright (c) 1993
// By: Bradford W. Mott
// Novermber 24,1993
//
///////////////////////////////////////////////////////////////////////////////
// $Log: M68681.hxx,v $
// Revision 1.2  1995/01/13  00:32:06  bmott
// Removed the inline prefix from several member functions that shouldn't
// have been inlined
//
// Revision 1.1  1994/02/18  20:12:12  bmott
// Initial revision
//
///////////////////////////////////////////////////////////////////////////////

#ifndef M68681_HXX
#define M68681_HXX

#include <sys/types.h>
#include "BasicDevice.hxx"

class M68681 : public BasicDevice {
  private:
    unsigned char MR1A;                    // Mode register 1 A
    unsigned char MR2A;                    // Mode register 2 A
    unsigned char SRA;                     // Status register A
    unsigned char CSRA;                    // Clock-select register A
    unsigned char CRA;                     // Command register A
    unsigned char RBA;                     // Receiver buffer A
    unsigned char TBA;                     // Transmitter buffer A
    unsigned char IPCR;                    // Input port change register

    unsigned char ACR;                     // Auxiliary control register
    unsigned char ISR;                     // Interrupt status register
    unsigned char IMR;                     // Interrupt mask register
    unsigned char CUR;                     // Counter mode: counter MSB
    unsigned char CTUR;                    // Counter/timer uppper register
    unsigned char CLR;                     // Counter mode: counter LSB
    unsigned char CTLR;                    // Counter/timer lower register

    unsigned char MR1B;                    // Mode register 1 B
    unsigned char MR2B;                    // Mode register 2 B
    unsigned char SRB;                     // Status register B
    unsigned char CSRB;                    // Clock-select register B
    unsigned char CRB;                     // Command register B
    unsigned char RBB;                     // Receiver buffer B
    unsigned char TBB;                     // Transmitter buffer B

    unsigned char IVR;                     // Interrupt-vector register

    unsigned char mr1a_pointer;            // Determines MR1A/MR2A
    unsigned char mr1b_pointer;            // Determines MR1B/MR2B

    unsigned char receiver_a_state;        // State of receiver A
    unsigned char transmitter_a_state;     // State of transmitter A
    unsigned char receiver_b_state;        // State of receiver B
    unsigned char transmitter_b_state;     // State of transmitter B

    static long baudrate_table[32];        // Table of times for baud rates
   
    int coma_read_id;                      // Pipe to command for port a
    int coma_write_id;                     // Pipe to command for port a
    int comb_read_id;                      // Pipe to command for port b
    int comb_write_id;                     // Pipe to command for port b
    pid_t coma_pid;                        // Proccess ID for port a command
    pid_t comb_pid;                        // Proccess ID for port b command
 

    unsigned long base_address;             // Base address of the DUART
    unsigned long offset_to_first_register; // Offset to the first registers
    unsigned long offset_between_registers; // Offset to between registers
    unsigned long interrupt_level;          // The interrupt level sent to CPU

    int  StartPortCommand(String command, int std_flag, 
             int &read, int &write, pid_t &pid);
    void SetInterruptStatusRegister();
 
  public:
    M68681(String args, BasicCPU *cpu);

    ~M68681();

    // See if the address maps into the device (1=Yes,0=No)
    char CheckMapped(unsigned long addr);

    // Return the lowest address used by the device
    virtual unsigned long LowestAddress()
    { return(base_address); }

    // Return the highest address used by the device
    virtual unsigned long HighestAddress()
    { 
      return(base_address+offset_to_first_register+15*offset_between_registers);
    }

    // Get a byte from the device
    virtual unsigned char Peek(unsigned long addr);

    // Put a byte into the device
    virtual void Poke(unsigned long addr, unsigned char c);

    // Reset handler
    virtual void Reset();

    // The BasicDevice's InterruptRequest routine is fine for us!!!

    // The BasicDevice's InterruptAcknowledge does not handle vectored
    // interrupts so well have to change it!!!
    long InterruptAcknowledge(int level);
   
    // Handle the DUART's events
    void EventCallback(long type, void* pointer);
};

#endif
